/*
   CRT - Trinitron - HiRes
   
   Copyright (C) 2019 guest(r) - guest.r@gmail.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#define brightboost 1.50
#define gammaOUT    0.5
#define scanline    8.0      // scanline param, vertical sharpness
#define beam_min    1.30     // dark area beam min - narrow
#define beam_max    1.10     // bright area beam max - wide
#define saturation  1.1
#define h_sharp     0.75     // lower values are sharper
#define v_sharp     0.375    // lower values are sharper


const static float3 dt = float3(1.0,1.0,1.0);


// The name of this effect
string name : NAME = "CrtGuest";

float2 ps                       : TEXELSIZE;

float4x4 World                  : WORLD;
float4x4 View                   : VIEW;
float4x4 Projection             : PROJECTION;
float4x4 Worldview              : WORLDVIEW;               // world * view
float4x4 ViewProjection         : VIEWPROJECTION;          // view * projection
float4x4 WorldViewProjection    : WORLDVIEWPROJECTION;     // world * view * projection

string combineTechique          : COMBINETECHNIQUE = "CrtGuest";

texture SourceTexture	        : SOURCETEXTURE;
texture WorkingTexture          : WORKINGTEXTURE;

sampler	decal = sampler_state
{
	Texture	  = (SourceTexture);
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};


struct out_vertex {
	float4 position : POSITION;
	float4 color    : COLOR;
	float2 t0       : TEXCOORD0;
}; 
 

out_vertex  VS_VERTEX(float3 position : POSITION, float2 texCoord : TEXCOORD0 )
{ 
	out_vertex OUT = (out_vertex)0;

	OUT.position = mul(float4(position,1.0),WorldViewProjection);
	OUT.t0 = texCoord;
	return OUT;  
}

float3 SoftLite (float2 pos)
{
	float4 yx = 0.50*float4(ps, -ps)*float4(1.0,0.5,1.0,0.5)*float4(h_sharp,v_sharp,h_sharp,v_sharp);
	float4 xy = 1.00*float4(ps, -ps)*float4(1.0,0.5,1.0,0.5)*float4(h_sharp,v_sharp,h_sharp,v_sharp);
	
	float3 c11 = tex2D(decal, pos        ).xyz; c11*=c11;
	float3 c00 = tex2D(decal, pos + xy.zw).xyz; c00*=c00; 
	float3 c20 = tex2D(decal, pos + xy.xw).xyz; c20*=c20; 
	float3 c22 = tex2D(decal, pos + xy.xy).xyz; c22*=c22; 
	float3 c02 = tex2D(decal, pos + xy.zy).xyz; c02*=c02; 
	float3 s00 = tex2D(decal, pos + yx.zw).xyz; s00*=s00; 
	float3 s20 = tex2D(decal, pos + yx.xw).xyz; s20*=s20; 
	float3 s22 = tex2D(decal, pos + yx.xy).xyz; s22*=s22; 
	float3 s02 = tex2D(decal, pos + yx.zy).xyz; s02*=s02; 

	float d1=dot(abs(c00-c22),dt)+0.001;
	float d2=dot(abs(c20-c02),dt)+0.001;
	float m1=dot(abs(s00-s22),dt)+0.001;
	float m2=dot(abs(s02-s20),dt)+0.001;

	float3 t2=(d1*(c20+c02)+d2*(c00+c22))/(2.0*(d1+d2));
	
	return .25*(c11+t2+(m2*(s00+s22)+m1*(s02+s20))/(m1+m2));
}

float3 sw(float x, float3 color)
{
	float3 tmp = lerp(2.75*float3(beam_min,beam_min,beam_min),float3(beam_max,beam_max,beam_max), color);
	tmp = lerp(float3(beam_max,beam_max,beam_max), tmp, pow(float3(x,x,x),float3(0.3,0.3,0.3)+color));
	float3 ex = float3(x,x,x)*tmp;
	return max(exp2(-scanline*ex*ex)/(0.6 + 0.4*color), 0.0001);
}

float4 PS_FRAGMENT (in out_vertex VAR) : COLOR
{	
	float2 TextureSize = float2(1.0/ps.x, 1.0/ps.y);
	
// Calculating texel coordinates

	float3 color = SoftLite (VAR.t0);
	
// calculating scanlines
	
	float f = frac(VAR.t0.y * 0.5*TextureSize.y); f = f - 0.5;
	color*= sw(abs(f),color) + sw(f+1.0,color) + sw(abs(f-1.0),color);
	color*=brightboost;
	color = min(color,1.0);
	
	color = pow(color, float3(gammaOUT,gammaOUT,gammaOUT));
	
	float l = length(color);
	color = normalize(pow(color + 0.0000001, float3(saturation,saturation,saturation)))*l;	
	
	return float4(color,1.0);	
}


//
// Technique
//

technique CrtGuest
{
    pass P0
    {
        // shaders		
        VertexShader = compile vs_3_0 VS_VERTEX();
        PixelShader  = compile ps_3_0 PS_FRAGMENT(); 
    }  
}

